package Kirjalista;

import java.io.PrintStream;
import java.util.Random;

/**
 * Genre-luokka
 * - Sisältää genren tiedot
 * - Muuttaa merkkijonon genren tiedoiksi
 * - Muuttaa genren tiedot merkkijonoksi
 */
public class Genre {
    
    private int id = 0;
    private String nimi = "";
    
    private static int seuraavaId = 1;
    
    /**
     * Ottaa merkkijonon ja muuttaa sen genren tiedoiksi, poistaa turhat völilyönnit ja muuttaa
     * syötteen pieniksi kirjaimiksi
     * @param mjono Merkkijono, josta genren tiedot saadaan
     * <pre name="test">
     * Genre g = new Genre();
     * g.parse("fantasia");
     * g.toString() === "fantasia";
     * g.parse("FANTASIA");
     * g.toString() === "fantasia";
     * g.parse("  Fantasia   ");
     * g.toString() === "fantasia";
     * </pre>
     */
    public void parse(String mjono) {
        this.nimi = mjono.trim().toLowerCase();
    }
    
    
    /**
     * Palauttaa genren nimen merkkijonona
     */
    @Override
    public String toString() {
        return (this.nimi);
    }
    
    
    /**
     * Rekisteröi genren antamalla sille id:n, kasvattaa seuraavaId:tä
     * <pre name="test">
     *  Genre g1 = new Genre();
     *  Genre g2 = new Genre();
     *  g1.rekisteroi();
     *  g2.rekisteroi();
     *  g1.getId() === g2.getId() - 1;
     *  </pre>
     */
    public void rekisteroi() {
        this.id = seuraavaId;
        seuraavaId++;
    }
    
    
    
    private String[] esim = {"Fantasia", "Sci-fi", "Kauhu"};
    /**
     * Asettaa genreksi jonkun ylläolevista
     * TODO poista
     */
    public void esimerkkiGenre() {
        Random rand = new Random();
        this.nimi = esim[rand.nextInt(3)];
    }
    
    /**
     * Luo olion testejä varten
     * @return Genre satunnaisilla arvoilla
     */
    public static Genre testiGenre() {
        Genre g = new Genre();
        Random rand = new Random();
        g.nimi = g.esim[rand.nextInt(3)];
        g.rekisteroi();
        return g;
    }
    
    /**
     * Palauttaa genren nimen
     * @return Genren nimi
     */
    public String getNimi() {
        return this.nimi;
    }
    
    /**
     * Palauttaa genren id:n
     * @return Genren id
     */
    public int getId() {
        return this.id;
    }
    
    
    /**
     * Tulostaa genren tiedot
     * @param out Tietovirta johon tulostetaan
     */
    public void tulosta(PrintStream out) {
        out.println(this.id + " " + this.nimi);
    }


    /**
     * Asettaa id:lle annetun arvon
     * @param id arvo, joka asetetaan id:ksi
     * Tämä olemassa, koska tietyllä genrellä on aina tietty id ja ilman tätä aliohjelmaa tulisi
     * ongelmia sellaisten lisättyjen kirjojen kanssa, joiden genre on jo "tietokannassa." Ilman tätä
     * kyseisten genrejen id olisi 0 ja ohjelma ei osaisi hakea niitä
     */
    public void setId(int id) {
        this.id = id;
    }
}
