package Kirjalista;

import java.util.ArrayList;
import java.util.NoSuchElementException;

/**
 * Monikkoluokka genreille
 * - Sisältää listan genreistä
 * - Lisää ja poistaa kirjoja
 * - Lukee ja kirjoittaa tiedostoon
 * - Etsii ja lajittelee
 * <pre name="test">
 * #import java.util.NoSuchElementException;
 * Genret g = new Genret();
 * Genre g1 = Genre.testiGenre();
 * Genre g2 = Genre.testiGenre();
 * Genre g3 = Genre.testiGenre();
 * 
 * g.lisaa(g1);
 * g.lisaa(g2);
 * g.lisaa(g3);
 * 
 * Genre g4 = new Genre();
 * g4.parse(g1.toString());
 * 
 * g.onOlemassa(g4.getNimi()) === true;
 * 
 * g.haeIdNimella(g1.getNimi()) === g1.getId();
 * g.haeIdNimella(g2.getNimi()) === g2.getId();
 * g.haeIdNimella(g3.getNimi()) === g3.getId();
 * 
 * g.hae(g1.getId()).toString() === g1.getNimi();
 * g.hae(g2.getId()).toString() === g2.getNimi();
 * g.hae(g3.getId()).toString() === g3.getNimi(); 
 * 
 * int g1Id = g1.getId(); 
 * g.poista(g1Id);
 * g.hae(g1Id); #THROWS NoSuchElementException
 * </pre>
 */
public class Genret {
    private ArrayList<Genre> alkiot = new ArrayList<Genre>();
    
    
    /**
     * Lisää genren listaan, jos samannimistä genreä ei ole olemassa
     * @param genre Lisättävä genre
     */
    public void lisaa(Genre genre) {
        if (onOlemassa(genre.getNimi())) {
            genre.setId(haeIdNimella(genre.getNimi()));
            return;
        }
        genre.rekisteroi();
        alkiot.add(genre);
    }
    
    
    /**
     * Hakee genren id:n nimen perusteella
     * @param nimi Nimi, jonka perusteella id haetaan
     * @return Genren id, -1 jos ei löydy
     */
    public int haeIdNimella(String nimi) {
        for (int i = 0; i < alkiot.size(); i++) {
            if (alkiot.get(i).getNimi().toLowerCase().contentEquals(nimi.toLowerCase())) return alkiot.get(i).getId();
        }
        return -1;
    }


    /**
     * Poistaa genren annetulla id:llä
     * @param id Id, jonka perusteella genre poistetaan
     * TODO testit
     */
    public void poista(int id) {
        for (int i = 0; i < alkiot.size(); i++) {
            if (alkiot.get(i).getId() == id) {
                alkiot.remove(i);
                return;
            }
        }
        throw new NoSuchElementException("Genreä id:llä " + id + " ei löydy.");
    }
    
    
    /**
     * Tarkistaa, onko tietyn nimistä genreä olemassa
     * @param nimi Genren nimi
     * @return True, jos genre on olemassa
     */
    public boolean onOlemassa(String nimi) {
        for (int i = 0; i < alkiot.size(); i++) {
            if (alkiot.get(i).getNimi().toLowerCase().contentEquals(nimi.toLowerCase())) {
                return true;
                
            }
        }
        return false;
    }
    
    
    /**
     * Hakee genren nimen id:n perusteella
     * @param id haettavan genren id
     * @return Viite genreen halutulla id:llä
     * TODO testit
     */
    public Genre hae(int id) {
        for (int i = 0; i < alkiot.size(); i++) {
            if (alkiot.get(i).getId() == id) return alkiot.get(i);
        }
        
        throw new NoSuchElementException("Genreä id:llä " + id + " ei löydy!");
    }
    
}
